@@ -1,7 +1,9 @@
 """Orchestrates view.py and model.py"""
 
 import os
+import sys
 import time
+import json
 import errno
 import shutil
 import logging
@@ -43,7 +45,7 @@ class State(dict):
 
     """
 
-    def __init__(self, ctrl, storage):
+    def __init__(self, ctrl, storage, parent_environ=None):
         super(State, self).__init__({
             "profileName": storage.value("startupProfile"),
             "appRequest": storage.value("startupApplication"),
@@ -66,6 +68,12 @@ class State(dict):
             # Cache, for performance only
             "rezEnvirons": {},
 
+            # Parent environment for all applications
+            "parentEnviron": parent_environ or {},
+
+            # Cache environment testing result
+            "testedEnvirons": {},
+
             "rezApps": odict(),
             "fullCommand": "rez env",
             "serialisationMode": (
@@ -190,6 +198,8 @@ class Controller(QtCore.QObject):
 
     patch_changed = QtCore.Signal(str)  # full patch string
 
+    running_cmd_updated = QtCore.Signal(int)
+
     states = [
         _State("booting", help="ALLZPARK is booting, hold on"),
         _State("resolving", help="Rez is busy resolving a context"),
@@ -203,10 +213,16 @@ class Controller(QtCore.QObject):
         _State("pkgnotfound", help="One or more packages was not found"),
     ]
 
-    def __init__(self, storage, stdio=None, stderr=None, parent=None):
+    def __init__(self,
+                 storage,
+                 parent_environ=None,
+                 stdio=None,
+                 stderr=None,
+                 parent=None):
+
         super(Controller, self).__init__(parent)
 
-        state = State(self, storage)
+        state = State(self, storage, parent_environ)
 
         models = {
             "apps": model.ApplicationModel(),
@@ -217,6 +233,8 @@ class Controller(QtCore.QObject):
             "packages": model.PackagesModel(self),
             "context": model.ContextModel(),
             "environment": model.EnvironmentModel(),
+            "parentenv": model.EnvironmentModel(),
+            "diagnose": model.EnvironmentModel(),
             "commands": model.CommandsModel(),
         }
 
@@ -227,6 +245,8 @@ class Controller(QtCore.QObject):
         timers["commandsPoller"].timeout.connect(self.on_tasks_polled)
         timers["commandsPoller"].start(500)
 
+        models["parentenv"].load(state["parentEnviron"].copy())
+
         # Initialize the state machine
         self._machine = transitions.Machine(
             model=state,
@@ -257,6 +277,10 @@ class Controller(QtCore.QObject):
     def models(self):
         return self._models
 
+    @property
+    def timers(self):
+        return self._timers
+
     @property
     def state(self):
         return self._state
@@ -280,6 +304,18 @@ class Controller(QtCore.QObject):
     def context(self, app_request):
         return self._state["rezContexts"][app_request].to_dict()
 
+    def parent_environ(self):
+        environ = self._state["parentEnviron"].copy()
+        # Inject user environment
+        #
+        # NOTE: Rez takes precendence on environment, so a user
+        # cannot edit the environment in such a way that packages break.
+        # However it also means it cannot edit variables also edited
+        # by a package. Win some lose some
+        environ = dict(environ, **self._state.retrieve("userEnv", {}))
+
+        return environ
+
     def environ(self, app_request):
         """Fetch the environment of a context
 
@@ -305,15 +341,18 @@ class Controller(QtCore.QObject):
             return env[app_request]
 
         except KeyError:
+            context = ctx[app_request]
+            parent_env = self.parent_environ()
             try:
-                environ = ctx[app_request].get_environ()
-                env[app_request] = environ
-                return environ
+                environ = context.get_environ(parent_environ=parent_env)
 
             except rez.ResolvedContextError:
                 return {
                     "error": "Failed context"
                 }
+            else:
+                env[app_request] = environ
+                return environ
 
     def resolved_packages(self, app_request):
         return self._state["rezContexts"][app_request].resolved_packages
@@ -323,7 +362,8 @@ class Controller(QtCore.QObject):
     # ----------------
 
     def on_tasks_polled(self):
-        self._models["commands"].poll()
+        running_count = self._models["commands"].poll()
+        self.running_cmd_updated.emit(running_count)
 
     def on_state_changed(self):
         state = self._name_to_state[self._state.state]
@@ -466,11 +506,11 @@ class Controller(QtCore.QObject):
 
             yield pkg
 
-    def env(self, request, use_filter=True):
+    def env(self, requests, use_filter=True):
         """Resolve context, relative Allzpark state
 
         Arguments:
-            request (str): Fully formatted request, including any
+            requests (list): Fully formatted request, including any
                 number of packages. E.g. "six==1.2 PySide2"
             use_filter (bool, optional): Whether or not to apply
                 the current package_filter
@@ -481,7 +521,7 @@ class Controller(QtCore.QObject):
         paths = self._package_paths()
 
         return rez.env(
-            request,
+            requests,
             package_paths=paths,
             package_filter=package_filter if use_filter else None
         )
@@ -684,6 +724,8 @@ class Controller(QtCore.QObject):
                 "command", app_model.data(app_index, "tool"))
             is_detached = kwargs.get(
                 "detached", app_model.data(app_index, "detached"))
+            stdout = kwargs.get("stdout", self.info)
+            stderr = kwargs.get("stderr", self.error)
 
             assert tool_name, (
                 "There should have been at least one tool name. "
@@ -692,7 +734,7 @@ class Controller(QtCore.QObject):
 
             overrides = self._models["packages"]._overrides
             disabled = self._models["packages"]._disabled
-            environ = self._state.retrieve("userEnv", {})
+            environ = self.parent_environ()
 
             self.debug(
                 "Launching %s%s.." % (
@@ -714,8 +756,8 @@ class Controller(QtCore.QObject):
                 parent=self
             )
 
-            cmd.stdout.connect(self.info)
-            cmd.stderr.connect(self.error)
+            cmd.stdout.connect(stdout)
+            cmd.stderr.connect(stderr)
             cmd.error.connect(on_error)
 
             cmd.execute()
@@ -809,7 +851,7 @@ class Controller(QtCore.QObject):
                 profiles = root()
 
             except Exception:
-                if log.level == logging.DEBUG:
+                if log.level < logging.INFO:
                     traceback.print_exc()
 
                 self.error("Could not find profiles in %s" % root)
@@ -831,11 +873,13 @@ class Controller(QtCore.QObject):
         self._models["apps"].reset()
         self._models["context"].reset()
         self._models["environment"].reset()
+        self._models["diagnose"].reset()
         self._models["packages"].reset()
         self._models["profileVersions"].setStringList([])
 
         self._state["rezContexts"].clear()
         self._state["rezEnvirons"].clear()
+        self._state["testedEnvirons"].clear()
         self._state["rezApps"].clear()
 
         def on_apps_found(apps):
@@ -923,16 +967,19 @@ class Controller(QtCore.QObject):
             context = self.context(app_request)
             environ = self.environ(app_request)
             packages = self.resolved_packages(app_request)
+            diagnose = self._state["testedEnvirons"].get(app_request, {})
 
         except Exception:
             self._models["packages"].reset()
             self._models["context"].reset()
             self._models["environment"].reset()
+            self._models["diagnose"].reset()
             raise
 
         self._models["packages"].reset(packages)
         self._models["context"].load(context)
         self._models["environment"].load(environ)
+        self._models["diagnose"].load(diagnose)
 
         tools = self._models["apps"].find(app_request)["tools"]
         self._state["tool"] = tools[0]
@@ -1123,6 +1170,32 @@ class Controller(QtCore.QObject):
 
         return pixmap
 
+    def shell_code(self):
+        app_request = self._state["appRequest"]
+        context = self._state["rezContexts"][app_request]
+        parent_env = self.parent_environ()
+        return context.get_shell_code(parent_environ=parent_env)
+
+    def test_environment(self):
+        app_request = self._state["appRequest"]
+
+        command = (
+            '%s -c "'
+            'import os,sys,json;'
+            'sys.stdout.write(json.dumps(os.environ.copy(),ensure_ascii=0))"'
+        ) % sys.executable
+
+        def load(message):
+            try:
+                env = json.loads(message)
+            except json.JSONDecodeError:
+                self.info(message)  # regular messages during resolve
+            else:
+                self._state["testedEnvirons"][app_request] = env
+                self._models["diagnose"].load(env)
+
+        self.launch(command=command, stdout=load)
+
 
 class Command(QtCore.QObject):
     stdout = QtCore.Signal(str)
@@ -1190,21 +1263,20 @@ class Command(QtCore.QObject):
             "command": self.cmd,
             "stdout": subprocess.PIPE,
             "stderr": subprocess.PIPE,
-            "parent_environ": None,
+            "parent_environ": self.environ or None,
             "startupinfo": startupinfo
         }
+        if rez.project == "rez":
+            # bleeding-rez adds `universal_newlines=True` when spawning shell,
+            # nerdvegas/rez doesn't.
+            kwargs["universal_newlines"] = True
+
+        if sys.version_info[:2] >= (3, 6):
+            kwargs["encoding"] = allzparkconfig.subprocess_encoding()
+            kwargs["errors"] = allzparkconfig.unicode_decode_error_handler()
 
         context = self.context
 
-        if self.environ:
-            # Inject user environment
-            #
-            # NOTE: Rez takes precendence on environment, so a user
-            # cannot edit the environment in such a way that packages break.
-            # However it also means it cannot edit variables also edited
-            # by a package. Win some lose some
-            kwargs["parent_environ"] = dict(os.environ, **self.environ)
-
         try:
             self.popen = context.execute_shell(**kwargs)
         except Exception as e:
